/*
 * Copyright 2021 Esri
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.example.app

import android.os.Bundle
import androidx.appcompat.app.AppCompatActivity

import android.graphics.Color
import android.util.Log
import android.widget.SearchView
import android.widget.Toast

import com.esri.arcgisruntime.ArcGISRuntimeEnvironment
import com.esri.arcgisruntime.mapping.ArcGISMap
import com.esri.arcgisruntime.mapping.BasemapStyle
import com.esri.arcgisruntime.mapping.Viewpoint
import com.esri.arcgisruntime.mapping.view.Graphic
import com.esri.arcgisruntime.mapping.view.GraphicsOverlay
import com.esri.arcgisruntime.mapping.view.MapView
import com.esri.arcgisruntime.symbology.SimpleMarkerSymbol
import com.esri.arcgisruntime.symbology.TextSymbol
import com.esri.arcgisruntime.tasks.geocode.GeocodeParameters
import com.esri.arcgisruntime.tasks.geocode.GeocodeResult
import com.esri.arcgisruntime.tasks.geocode.LocatorTask

import com.example.app.databinding.ActivityMainBinding

class MainActivity : AppCompatActivity() {

    private val activityMainBinding by lazy {
        ActivityMainBinding.inflate(layoutInflater)
    }

    private val mapView: MapView by lazy {
        activityMainBinding.mapView
    }

    private val graphicsOverlay: GraphicsOverlay by lazy {
        GraphicsOverlay()
    }

    private val locatorTask = LocatorTask("https://geocode-api.arcgis.com/arcgis/rest/services/World/GeocodeServer")

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(activityMainBinding.root)

        setApiKeyForApp()

        setupMap()

        setupSearchViewListener()

    }

    override fun onPause() {
        mapView.pause()
        super.onPause()
    }

    override fun onResume() {
        super.onResume()
        mapView.resume()
    }

    override fun onDestroy() {
        mapView.dispose()
        super.onDestroy()
    }

    private fun setApiKeyForApp(){
        // set your API key
        // Note: it is not best practice to store API keys in source code. The API key is referenced
        // here for the convenience of this tutorial.

        ArcGISRuntimeEnvironment.setApiKey("YOUR_API_KEY")

    }

    // set up your map here. You will call this method from onCreate()
    private fun setupMap() {
        // create a map with the BasemapStyle topographic
        val arcGISmap = ArcGISMap(BasemapStyle.ARCGIS_TOPOGRAPHIC)

        mapView.apply {
            // set the map to be displayed in the layout's MapView
            map = arcGISmap

            // set the viewpoint, Viewpoint(latitude, longitude, scale)
            setViewpoint(Viewpoint(34.0270, -118.8050, 72000.0))

            graphicsOverlays.add(graphicsOverlay)
        }

    }

    private fun setupSearchViewListener() {

        activityMainBinding.searchView.setOnQueryTextListener(object : SearchView.OnQueryTextListener {

            override fun onQueryTextChange(newText: String): Boolean {
                return false
            }

            override fun onQueryTextSubmit(query: String): Boolean {

                performGeocode(query)

                return false
            }

        })
    }

    private fun performGeocode(query: String) {

        val geocodeParameters = GeocodeParameters().apply {
            resultAttributeNames.add("*")
            maxResults = 1
            outputSpatialReference = mapView.spatialReference
        }

        val geocodeResultFuture = locatorTask.geocodeAsync(query, geocodeParameters)

        geocodeResultFuture.addDoneListener {
            try {
                val geocodeResult = geocodeResultFuture.get()
                if (geocodeResult.isNotEmpty()) {
                    displayResult(geocodeResult[0])

                } else {
                    Toast.makeText(this, "No results found.", Toast.LENGTH_LONG).show()
                }
            } catch (e: Exception) {
                Log.e(MainActivity::class.simpleName, "Error getting result" + e.message)
            }
        }

    }

    private fun displayResult(geocodeResult: GeocodeResult) {

        // clear the overlay of any previous result
        graphicsOverlay.graphics.clear()

        // create a graphic to display the address text
        val textSymbol = TextSymbol(
            18f,
            geocodeResult.label,
            Color.BLACK,
            TextSymbol.HorizontalAlignment.CENTER,
            TextSymbol.VerticalAlignment.BOTTOM
        )

        val textGraphic = Graphic(geocodeResult.displayLocation, textSymbol)
        graphicsOverlay.graphics.add(textGraphic)

        // create a graphic to display the location as a red square
        val simpleMarkerSymbol =
            SimpleMarkerSymbol(SimpleMarkerSymbol.Style.SQUARE, Color.RED, 12.0f)

        val markerGraphic =
            Graphic(geocodeResult.displayLocation, geocodeResult.attributes, simpleMarkerSymbol)
        graphicsOverlay.graphics.add(markerGraphic)

        mapView.setViewpointCenterAsync(geocodeResult.displayLocation)

    }

}

